
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/xv/xvCore.h>
#include <X11/xv/xvRegion.h>
#include <X11/xv/xvGC.h>
#include <X11/xv/xvGraphic.h>
#include <X11/xv/xvBasicBitmap.h>
#include <X11/xv/xvWindow.h>
#include <X11/xv/xvGrip.h>
#include <X11/xv/xvSizer.h>


// ---------------------------------------------

xvSizer::xvSizer()
{
}

// ---------------------------------------------

xvSizer::xvSizer(xvWindow & wparent, int x, int y, int wid, int hgt, int mode, int gripsize,
			XtCallbackProc size_callback, XtCallbackProc move_callback,
			XtEventHandler motion_callback, XtEventHandler stop_callback)
{
	init(wparent, x, y, wid, hgt, mode, gripsize, 
		size_callback, move_callback, motion_callback, stop_callback);
}

// ---------------------------------------------

xvSizer::~xvSizer()
{
}

// ---------------------------------------------

void xvSizer::init(xvWindow & wparent, int x, int y, int wid, int hgt, int mode, int gripsize,
			XtCallbackProc size_callback, XtCallbackProc move_callback,
			XtEventHandler motion_callback, XtEventHandler stop_callback)
{
	Visible=FALSE;
	Sizing=FALSE;
	Moving=FALSE;
	wide=wid; high=hgt;
	sizermode=mode;
	gsize=gripsize;
	minwidth=3*gsize+2;
	minheight=2*gsize+1;

	if (wide<minwidth) wide=minwidth;
	if (high<minheight) high=minheight;
	px=x; py=y;
	size_call=size_callback;
	move_call=move_callback;
	motion_call=motion_callback;
	stop_call=stop_callback;

	wparent.AddEvent(Button1MotionMask, motion_callback, (XtPointer) "move");
	wparent.AddEvent(ButtonReleaseMask, stop_callback, (XtPointer) "up");
}

// ---------------------------------------------

void xvSizer::setColours(xvWindow & wparent)
{
	wparent.GCSetForeground(wparent.stringToPixel("navy"));
	wparent.GCSetBackground(wparent.stringToPixel("grey20"));
}

// ---------------------------------------------

void xvSizer::SizerUnhook(xvWindow & wparent)
{
	wparent.RemoveEvent(Button1MotionMask, motion_call, (XtPointer) "move");
	wparent.RemoveEvent(ButtonReleaseMask, stop_call, (XtPointer) "up");
}

// ---------------------------------------------

void xvSizer::SizerShow(xvWindow & wparent)
{
	int iw=(wide-3*gsize)/2;
	int ih=(high-2*gsize);

	tl.init("topleft", wparent, size_call);
		tl.SetVertDistance(py);
		tl.SetHorizDistance(px);	
		tl.SetShadowWidth(0);
		tl.SetSize(gsize, gsize);	
	tm.init("topmid", wparent, move_call);
		tm.SetHorizDistance(px+gsize+iw);
		tm.SetVertDistance(py);
		tm.SetShadowWidth(0);
		tm.SetSize(gsize, gsize);
	tr.init("topright", wparent, size_call);
		tr.SetHorizDistance(px+2*gsize+2*iw);
		tr.SetVertDistance(py);
		tr.SetShadowWidth(0);
		tr.SetSize(gsize, gsize);
	bl.init("bottomleft", wparent, size_call);
		bl.SetVertDistance(py+gsize+ih);
		bl.SetHorizDistance(px);
		bl.SetShadowWidth(0);
		bl.SetSize(gsize, gsize);
	bm.init("bottommid", wparent, move_call);
		bm.SetHorizDistance(px+gsize+iw);
		bm.SetVertDistance(py+gsize+ih);
		bm.SetShadowWidth(0);
		bm.SetSize(gsize, gsize);
	br.init("bottomright", wparent, size_call);
		br.SetHorizDistance(px+2*gsize+2*iw);
		br.SetVertDistance(py+gsize+ih); 
		br.SetShadowWidth(0);
		br.SetSize(gsize, gsize);

	setColours(wparent);

	wparent.GCSetFunction(GXxor);	
	wparent.GDrawRectangle(px,py, wide, high);		
	Visible=TRUE;
	maxx=wparent.GetWidth();
	maxy=wparent.GetHeight();
}

// -------------------------------------

void xvSizer::SizerHide(xvWindow & wparent)
{
	setColours(wparent);
	wparent.GCSetFunction(GXxor);	
	wparent.GDrawRectangle(px, py, wide, high);		

	wparent.GCSetFunction(GXcopy);	
	tl.Hide();
	tm.Hide();
	tr.Hide();
	bl.Hide();
	bm.Hide();
	br.Hide();
	Visible=FALSE;
}

// ------------------------------------

Boolean xvSizer::SizerIsVisible()
{
	return (Visible);
}

// ------------------------------------

void xvSizer::SizerMove(xvWindow & wparent, XtPointer client)
{
	if (sizermode==XVSIZEONLY) return;

	setColours(wparent);

	char * which=(char *) client;
	Moving=FALSE;
	if (strcmp(which, "topmid")==0) {
		pivotpoint=1;
		Moving=TRUE;
	}
	if (strcmp(which, "bottommid"	)==0) {
		pivotpoint=2;
		Moving=TRUE;
	}
	if (Moving) {
		SizerHide(wparent);
		wparent.GCSetFunction(GXxor);
		lx=wide;	
		ly=high;	
		wparent.GDrawRectangle(px, py, lx, ly); 	
	}
}

// -------------------------------------

void xvSizer::SizerSize(xvWindow & wparent, XtPointer client)
{
	if (sizermode==XVMOVEONLY) return;

	setColours(wparent);

	char *which=(char *) client;
	
	Sizing=FALSE;
	if (strcmp(which, "bottomright")==0) {
		pivotpoint=4;
		Sizing=TRUE;
	}
	if (strcmp(which, "bottomleft")==0) {
		pivotpoint=3;
		Sizing=TRUE;
	}
	if (strcmp(which, "topright")==0) {
		pivotpoint=2;
		Sizing=TRUE;
	}
	if (strcmp(which, "topleft")==0) {
		pivotpoint=1;
		Sizing=TRUE;
	}
			
	if (Sizing) {
		SizerHide(wparent);
		wparent.GCSetFunction(GXxor);
		lx=wide;	
		ly=high;	
		wparent.GDrawRectangle(px, py, lx, ly); 	
	}
}

// -------------------------------------

void xvSizer::SizerMotion(xvWindow & wparent)
{
	int dx, dy; 
	int vx, vy;
	int rx, ry; 
	int ax, ay;
	int provx, provy;
	unsigned int b;

	setColours(wparent);

	if (Moving) {
		wparent.GCSetFunction(GXxor);
		wparent.GDrawRectangle(px, py, lx, ly);
		wparent.GCPointerXY(&ax, &ay, &rx, &ry, &b);	
		switch(pivotpoint) {
			case 1:
				provx=ax-lx/2;
				provy=ay;
				break;
			case 2:
				provx=ax-lx/2;
				provy=ay-ly;
				break;
			default:
				break;
		}	
		Boolean rangeflag=FALSE;
		if ((provx >=0) && (provx+lx <maxx)) 
			px=provx;
		else
			rangeflag=TRUE;
		if ((provy>=0) && (provy+ly <maxy-1)) 
			py=provy;
		else
			rangeflag=TRUE;
		if (rangeflag) 
			switch(pivotpoint) {
				case 1:
					wparent.MovePointer(px+lx/2, py);
					break;
				case 2:
					wparent.MovePointer(px+lx/2, py+ly);
					break;
			}
		wparent.GDrawRectangle(px, py, lx, ly);
	}

	if (Sizing) {
		wparent.GCSetFunction(GXxor);
		wparent.GDrawRectangle(px, py, lx, ly);

		wparent.GCPointerXY(&ax, &ay, &rx, &ry, &b);	
		Boolean rangeflag=FALSE;

		if (ax < 0) { ax=0; rangeflag=TRUE; }
		if (ax > maxx-1) { ax=maxx-1; rangeflag=TRUE; }
		if (ay < 0) { ay=0; rangeflag=TRUE; }
		if (ay > maxy-1) { ay=maxy-1; rangeflag=TRUE; }

		if (rangeflag) wparent.MovePointer(ax, ay);

		switch(pivotpoint) {
			case 4:
				dx=ax-px;
				dy=ay-py;
				if (dx<minwidth) dx=minwidth;
				if (dy<minheight) dy=minheight;
				break;
			case 3:
				vx=px+lx;
				dx=lx-(ax-px);
				if (dx<minwidth) dx=minwidth;
				px=vx-dx;
				dy=ay-py;
				if (dy<minheight) dy=minheight;
				break;
			case 2:
				dx=ax-px;
				if (dx<minwidth) dx=minwidth;
				vy=py+ly;
				dy=ly-(ay-py);
				if (dy<minheight) dy=minheight;
				py=vy-dy;
				break;
			case 1:
				vy=py+ly;
				dy=ly-(ay-py);
				if (dy<minheight) dy=minheight;
				py=vy-dy;
				vx=px+lx;
				dx=lx-(ax-px);
				if (dx<minwidth) dx=minwidth;
				px=vx-dx;
				break;				
			default:
				break;
		}

		wparent.GDrawRectangle(px, py, dx, dy);
	
		lx=dx;
		ly=dy;	
	}
}

// ------------------------------------

void xvSizer::SizerStop(xvWindow & wparent) 
{
	if (Sizing || Moving) {
		setColours(wparent);
		wparent.GCSetFunction(GXxor);

		wparent.GDrawRectangle(px, py, lx, ly);
		wide=lx;
		high=ly;
		SizerShow(wparent);
		Sizing=FALSE;	
		Moving=FALSE;
	}
}

// --------------------------------------

void xvSizer::SizerInfo(int *x, int *y, int *wid, int *hgt)
{
	*x=px; *y=py;
	*wid=wide; *hgt=high;
}

// --------------------------------------

void xvSizer::SizerWarp(xvWindow & wparent, int wpx, int wpy, int wid, int hgt)
{
	if (Visible) SizerHide(wparent);
	px=wpx;
	py=wpy;
	wide=wid;
	high=hgt;
	if (Visible) SizerShow(wparent);
}	