
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

/* =============================================
	xvis user interface builder for Athena widgets using
	xv class library.

	Calls Segment

	V1.0BETA  - last edited 25/06/96
    ============================================= */

// ---------------------------------------------
// Include file declarations

#include "xvis.h"
#include <string.h>
#include <stdio.h>

// ---------------------------------------------
// Local data

int sel=0;
int local_mode=-1;
int ref_array[20];

// ---------------------------------------------
// local forward function declares

void OpenCallEditWindow();

// --------------------------------------------
// Callbacks

void func_calllist_call(Widget aw, XtPointer client, XtPointer call)
{
	XawListReturnStruct *val=(XawListReturnStruct *) call;
	sel=val->list_index;
	xvisCallList.Close();
	OpenCallEditWindow();
}

void func_callclose(Widget aw, XtPointer client, XtPointer call)
{
	xvisCallList.Close();
}

void editor_ok(Widget aw, XtPointer client, XtPointer call)
{
	char *temp=(char *) client;
	int which;
	sscanf(temp, "%d", &which);
	int obj=editor[which].object;
	int ref=editor[which].ref;
	int mode=editor[which].mode;	

	XtVaGetValues(editor[which].editor->xvEditField::ID(), XtNstring, &temp, NULL);

	switch(mode) {
		case CALLMODE:
			if (objects[obj].callbackcode[ref] != NULL) {
				delete objects[obj].callbackcode[ref];
				objects[obj].callbackcode[ref]=NULL;
			}
			objects[obj].callbackcode[ref]=new char[strlen(temp)+10];
			strcpy(objects[obj].callbackcode[ref], temp);
			break;
		case EVENTMODE:
			if (objects[obj].eventcode[ref] != NULL) {
				delete objects[obj].eventcode[ref];
				objects[obj].eventcode[ref]=NULL;
			}
			objects[obj].eventcode[ref]=new char[strlen(temp)+10];
			strcpy(objects[obj].eventcode[ref], temp);
			break;	
		default:
			break;
	}	
	
	delete editor[which].editor;
	delete editor[which].edit_buffer;
	editor[which].in_use=FALSE;
}

void editor_cancel(Widget aw, XtPointer client, XtPointer call)
{
	char *temp=(char *) client;
	int which;
	sscanf(temp, "%d", &which);
	delete editor[which].editor;
	delete editor[which].edit_buffer;
	editor[which].in_use=FALSE;
}

//---------------------------------------------
// Local Stuff

void buildCalls(String hdr)
{
	char **call_data=object_call_names[CurrentObjectType];
	int call_num=object_call_number[CurrentObjectType];

	for (int i=0; i<call_num; i++) {
		calls_list[num_calls]=call_data[i];
		ref_array[num_calls++]=i;
	}

	strcpy( hdr, "Call List");

}

void buildEvents(String hdr)
{
	for (int i=0; i<NUMEVENTS; i++)
		if ((objects[CurrentObject].events & eventinfo[i].eventbits) != 0) {
			calls_list[num_calls]=eventinfo[i].eventName;
			ref_array[num_calls++]=i;
		}

	strcpy(hdr, "Event List");
}

void buildCallsWindow(int mode)
{
	num_calls=0;
	char header[20];

	local_mode=mode;

	switch (mode) {
		case CALLMODE:
			buildCalls(header);
			break;
		case EVENTMODE:
			buildEvents(header);
			break;
		default:
			break;
	}

	xvisCallList.init(header, xvisControl);

	callform.init("Callform", xvisCallList);
	callform.SetBackground("gray85");
	
	callview.init("callview", callform, TRUE, FALSE, TRUE);
	callview.SetFixedPosition(TRUE);

	calllist.init("calllist", callview, func_calllist_call, 
		num_calls, calls_list, 1);
	calllist.SetHeight(200);
	calllist.SetWidth(200);
	calllist.SetFixedPosition(TRUE);

	callclose.init("Close", callform, func_callclose);
	callclose.SetWidth(80);
	callclose.SetHeight(30);
	callclose.SetRelVertical(callview);
	callclose.SetHorizDistance(60);
	callclose.SetShadowWidth(4);
	callclose.SetBeNiceToColormap(FALSE);	
	xvisCallList.Open();
		
}

// ---------------------------------------------

void OpenCallEditWindow()
{
	callType *calltype;
	char **callname;

	int obj_sel=ref_array[sel];
	int ptr=findFirstEditEntry();
	
	editor[ptr].in_use=TRUE;
	editor[ptr].object=CurrentObject;
	editor[ptr].mode=local_mode;
	editor[ptr].ref=obj_sel;
	sprintf(editor[ptr].client, "%d", ptr);

	switch(local_mode) {
		case CALLMODE:
			calltype=object_call_types[CurrentObjectType];
			callname=object_call_names[CurrentObjectType];

			sprintf(editor[ptr].name, "%s_%s", 
				objects[CurrentObject].name, callname[obj_sel]);

			editor[ptr].edit_buffer=new char[MAXEDITBUFFERSIZE];

			if (objects[CurrentObject].callbackcode[obj_sel]==NULL)
				switch(calltype[obj_sel]) {
					case XVCallback:
						sprintf(editor[ptr].edit_buffer, 
							"void %s (Widget aw, XtPointer clientdata, XtPointer calldata)\n{\n\n}\n", 
							editor[ptr].name);
						break;
					case XVTimeout:
						sprintf(editor[ptr].edit_buffer, 
							"void %s (XtPointer calldata, XtIntervalId *id)\n{\n\t%s.Reset();\n}\n", 
							editor[ptr].name, objects[CurrentObject].name);
						break;
				}
			else
				strcpy(editor[ptr].edit_buffer, objects[CurrentObject].callbackcode[obj_sel]);
  			
			break;
		case EVENTMODE:
			sprintf(editor[ptr].name, "%s_%s", 
				objects[CurrentObject].name, eventinfo[obj_sel].eventName);

			editor[ptr].edit_buffer=new char[MAXEDITBUFFERSIZE];

			if (objects[CurrentObject].eventcode[obj_sel]==NULL)
				sprintf(editor[ptr].edit_buffer, 
					"void %s (Widget aw, XtPointer client_data, %s *event, Boolean *cont)\n{\n\n}\n", 
						editor[ptr].name, eventinfo[obj_sel].eventStruct);
			else 
				strcpy(editor[ptr].edit_buffer, objects[CurrentObject].eventcode[obj_sel]);
			
			break;
	}
	editor[ptr].editor=new xvEditWindow( xvisContext, editor[ptr].name, 
		XawtextEdit, editor[ptr].edit_buffer, 600, 400, 
		editor_ok, editor_cancel, editor[ptr].client);
	editor[ptr].editor->xvEditField::ChangeFont("*times-*-r-*-140*");		
	editor[ptr].editor->SetWindowResource(XtNiconPixmap, 
			editor[ptr].editor->xvEditField::SetPixmapInfo(xvis_xpm, NULL));
	editor[ptr].editor->xvEditField::SetBackground("LemonChiffon");
	editor[ptr].editor->Open();

}

// ---------------------------------------------
// replace a string

char *replaceString(String source, String oldstring, String newstring, String dest)
{
	int p, q;

	for (int y=0; y<MAXEDITBUFFERSIZE; y++)
		dest[y]='\0';

	char *ptr=strstr(source, oldstring);
	if (ptr != NULL) {
		p=(int) (ptr-source);
		strncpy(dest, source, p);
		strcat(dest, newstring);	
		q=strlen(dest);
		for (int k=p+strlen(oldstring); k<strlen(source); k++, q++)
			dest[q]=source[k];
		dest[q]='\0';

		strcpy(source, dest);
		dest[0]='\0';
	}
	strcpy(dest, source);
}

// ---------------------------------------------
// replace old string in code

void updateCallbackCode(String oldname, String newname)
{
	char *sourcebuffer=new char[MAXEDITBUFFERSIZE];
	char *destbuffer=new char[MAXEDITBUFFERSIZE];

	for (int i=0; i<NUMEVENTS; i++)
		if (objects[prop_object_index].eventcode[i] != NULL) {
			strcpy(sourcebuffer, objects[prop_object_index].eventcode[i]);
			replaceString(sourcebuffer, oldname, newname, destbuffer);
			delete objects[prop_object_index].eventcode[i];
			objects[prop_object_index].eventcode[i]=new char[strlen(destbuffer)+10];
			strcpy(objects[prop_object_index].eventcode[i], destbuffer);
		}
				
	for (int i=0; i<3; i++)
		if (objects[prop_object_index].callbackcode[i] != NULL) {
			strcpy(sourcebuffer, objects[prop_object_index].callbackcode[i]);
			replaceString(sourcebuffer, oldname, 
				objects[prop_object_index].name, destbuffer);
			delete objects[prop_object_index].callbackcode[i];
			objects[prop_object_index].callbackcode[i]=new char [strlen(destbuffer)+10];
			strcpy(objects[prop_object_index].callbackcode[i], destbuffer);
		}

	delete sourcebuffer;
	delete destbuffer;
}

// ---------------------------------------------
		